/*
 * polygon.c: simple utilities for polygon data structures
 */
#include <stdio.h>

#define  POLYGON_C
#include "polygon.h"

polygon *new_n_polygons(int Num_polygons, int Num_vertices)
{
   char    *Subroutine_Name = "new_n_polygons()";
   polygon *Polygons;
   vertex  *Vertices;
   int      i;
   int      Index;

   if ((Polygons = (polygon *) malloc(Num_polygons * sizeof(polygon))) == NULL)
   {
      printf("%s -- can not allocate Polygons. Abort!\n", Subroutine_Name);
      exit(1);
   }

   if ((Vertices = (vertex *) malloc(Num_vertices * Num_polygons
                                     * sizeof(vertex))) == NULL)
   {
      printf("%s -- can not allocate Vertices. Abort!\n", Subroutine_Name);
      exit(1);
   }

   for (i = 0, Index = 0; i < Num_polygons; i++, Index += Num_vertices)
   {
#if 0
      Polygons[i].max_n = Num_vertices;
#endif
      Polygons[i].Vertices = &(Vertices[Index]);
   }

   return(Polygons);
}

void free_polygons(polygon *Polygons)
{
   vertex  *Vertices;

   Vertices = Polygons -> Vertices;
   free(Vertices);
   free(Polygons);

}

/*
 * poly_print: print Poly p to stdout, prefixed by the label str
 */

void polygon_print(char *str, polygon *p)
{
   int             i;

   printf("%s: %d sides\n", str, p->n);
   polygon_vertex_label("        ", p->mask);
   for (i = 0; i < p->n; i++)
   {
      printf("   v[%d] ", i);
      polygon_vertex_print("", &p->Vertices[i], p->mask);
   }
}

void polygon_vertex_label(char *str, unsigned long mask)
{
   printf("%s", str);
   if (mask & VERTEX_MASK(sx))
      printf("   sx  ");
   if (mask & VERTEX_MASK(sy))
      printf("   sy  ");
   if (mask & VERTEX_MASK(sz))
      printf("   sz  ");
   if (mask & VERTEX_MASK(sw))
      printf("   sw  ");
   if (mask & VERTEX_MASK(x))
      printf("   x   ");
   if (mask & VERTEX_MASK(y))
      printf("   y   ");
   if (mask & VERTEX_MASK(z))
      printf("   z   ");
   if (mask & VERTEX_MASK(u))
      printf("   u   ");
   if (mask & VERTEX_MASK(v))
      printf("   v   ");
   if (mask & VERTEX_MASK(q))
      printf("   q   ");
   if (mask & VERTEX_MASK(r))
      printf("   r   ");
   if (mask & VERTEX_MASK(g))
      printf("   g   ");
   if (mask & VERTEX_MASK(b))
      printf("   b   ");
   if (mask & VERTEX_MASK(nx))
      printf("   nx  ");
   if (mask & VERTEX_MASK(ny))
      printf("   ny  ");
   if (mask & VERTEX_MASK(nz))
      printf("   nz  ");
   printf("\n");
}

void polygon_vertex_print(char *str, vertex *v, unsigned long mask)
{
   printf("%s", str);
   if (mask & VERTEX_MASK(sx))
      printf(" %6.1f", v->sx);
   if (mask & VERTEX_MASK(sy))
      printf(" %6.1f", v->sy);
   if (mask & VERTEX_MASK(sz))
      printf(" %6.2f", v->sz);
   if (mask & VERTEX_MASK(sw))
      printf(" %6.2f", v->sw);
   if (mask & VERTEX_MASK(x))
      printf(" %6.2f", v->x);
   if (mask & VERTEX_MASK(y))
      printf(" %6.2f", v->y);
   if (mask & VERTEX_MASK(z))
      printf(" %6.2f", v->z);
   if (mask & VERTEX_MASK(u))
      printf(" %6.2f", v->u);
   if (mask & VERTEX_MASK(v))
      printf(" %6.2f", v->v);
   if (mask & VERTEX_MASK(q))
      printf(" %6.2f", v->q);
   if (mask & VERTEX_MASK(r))
      printf(" %6.4f", v->r);
   if (mask & VERTEX_MASK(g))
      printf(" %6.4f", v->g);
   if (mask & VERTEX_MASK(b))
      printf(" %6.4f", v->b);
   if (mask & VERTEX_MASK(nx))
      printf(" %6.3f", v->nx);
   if (mask & VERTEX_MASK(ny))
      printf(" %6.3f", v->ny);
   if (mask & VERTEX_MASK(nz))
      printf(" %6.3f", v->nz);
   printf("\n");
}

